==> bonkle-0.9.tar.gz: bonkle/source/bonklenet.rb <==
#!/usr/local/bin/ruby
# Copyright (c) 2001 Brian Fundakowski Feldman
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: bonklenet.rb,v 1.11 2001/03/05 07:53:35 green Exp $
require 'bonkle'
require 'socket'
require 'thread'

module Bonkle
	module Message
		def Message.read(socket)
			m = socket.sysread(2)
			if m.size != 2
				return nil
			end
			socket.sysread(m.unpack('n')[0])
		end
		def Message.write(socket, message)
			socket.syswrite([message.size].pack('n').concat(message))
		end
		SUCCESS = 0
		FAILURE = 1
		FATAL = 2
		@@ecodes = {
			SUCCESS	=> '',			# success
			FAILURE	=> 'error: ',	# failure
			FATAL	=> 'fatal: '	# fatal, close the connection
		}
		def Message.swrite(socket, ecode, cookie, message)
			msg = [cookie, ecode, @@ecodes[ecode]].join(' ').concat(message)
			socket.syswrite([msg.size].pack('n').concat(msg))
		end
	end
	class Server
		class User
			[:name, :words, :xwords, :status, :commandq].each {|att|
			  attr_reader att
			}
			class MaxConn < Exception; end
			class NameUsed < Exception; end
			def initialize(sock, game)
				@socket = sock
				@game = game
				@name = ''
				@words = []
				@xwords = []
				@status = :init
				@starttime = Time.now
				@commandq = Queue.new
				@stop = false
				@selectable = false
				@mutex = Mutex.new
				@cvar = ConditionVariable.new
				@mustrun = 0
				@selthread = Thread.new {
					begin
						until @status == :dead
							if select_wait(1)
								@selectable = true
								@mutex.synchronize {
									@mustrun += 1
									@cvar.signal
								}
								Thread.stop
							end
						end
					rescue EOFError => e
					end
				}
				@thread = Thread.new {
					begin
						write("info Hello from Bonkle #{Bonkle::VERSION}; " + 
						  "please introduce yourself.")
						until @status == :dead
							process_event
						end
					rescue EOFError => e
					ensure
						if @status == :run
							@game.remove_user(self)
						end
						@status = :dead
						@selthread.run
					end
				}
			end
			def points
				pn = 0
				@game.locked {
					@words.each {|word| pn += @game.points(word)}
				}
				pn
			end
			private
			def select_wait(time)
				select([@socket], nil, nil, time)
			end
			def read
				msg = Message.read(@socket)
				@selectable = false
				@mutex.synchronize {@mustrun -= 1}
				@selthread.run
				msg
			end
			def write(*args)
				Message.write(@socket, *args)
			end
			def swrite(*args)
				Message.swrite(@socket, *args)
			end
			def process_event
				@mutex.synchronize {
					if @mustrun == 0
						@cvar.wait(@mutex)
					end
				}

				until @commandq.empty?
					command = @commandq.shift
					case command[0]
					when :info
						write("info #{command[1]}")
					when :say
						write("say #{command[1]} #{command[2]}")
					when :shout
						write("shout #{command[1]} #{command[2]}")
					when :quit
						write("quit #{command[1]} #{command[2]}")
					when :timeover
						write("timeover #{command[1]}")
					when :newgame
						write("newgame #{command[1]}")
					when :goodbye
						begin
							write("goodbye " + command[1])
						ensure
							@socket.close
							if @status != :init
								@game.remove_user(self)
							end
							@status = :dead
							return
						end
					end
					@mutex.synchronize {@mustrun -= 1}
				end

				if !@selectable
					return
				end

				case @status
				when :init
					message = read
					if message =~ /^(\d+)\sname\s(.+)$/
					    ck = $1
						@name = $2
						if @name !~ /^[A-Za-z0-9_-]*$/ || @name.length > 20
							@name = ''
						end
					end
					if @name.empty?
						swrite(Message::FAILURE, ck, "need valid name")
						return
					else
						begin
							@game.add_user(self)
						rescue MaxConn => e
							swrite(Message::FAILURE, ck, e.to_s)
							@stop = true
							return
						rescue NameUsed => e
							swrite(Message::FAILURE, ck, e.to_s)
							return
						end
					end
					swrite(Message::SUCCESS, ck, "name set")
					@status = :run
				when :run
					message = read
					if message !~ /^(\d+)\s/
						swrite(Message::FATAL, 0, 'invalid command')
						goodbye('fatal protocol error encountered')
						return
					end
					ck = $1		# cookie will be used
					case $'
					when /^timeleft(\s|$)/
						swrite(Message::SUCCESS, ck,
						  format("%.4f", @game.endtime - Time.now))
					when /^fboard(\s|$)/
						swrite(Message::SUCCESS, ck,
						  "pre-formatted Bonkle board:\n" +
						  @game.locked {@game.board.to_s})
					when /^boardsize(\s|$)/
						swrite(Message::SUCCESS, ck, @game.board.width.to_s)
					when /^board(\s|$)/
						swrite(Message::SUCCESS, ck,
						  @game.locked {@game.board.dice_array.join(' ')})
					when /^lusers(\s|$)/
						swrite(Message::SUCCESS, ck, @game.usernames.join(' '))
					when /^lwords(\s(.+)|$)/
						@game.crossout
						if $1.empty? || $2 == @name
							swrite(Message::SUCCESS, ck,
							  @game.locked {@words.sort.join(' ')})
						else
							msg = @game.crossout {|done|
								@game.user_do($2) {|u|
									if !done
										[Message::FAILURE, ck,
										  'game not finished']
									elsif !u
										[Message::FAILURE, ck, 'invalid user']
									else
										[Message::SUCCESS, ck,
										  u.words.sort.join(' ')]
									end
								}
							}
							swrite(*msg)
						end
					when /^lxwords(\s(.+)|$)/
						@game.crossout
						if $1.empty? || $2 == @name
							swrite(Message::SUCCESS, ck,
							  @game.locked {@xwords.sort.join(' ')})
						else
							msg = @game.crossout {|done|
								@game.user_do($2) {|u|
									if !done
										[Message::FAILURE, ck,
										  'game not finished']
									elsif !u
										[Message::FAILURE, ck, 'invalid user']
									else
										[Message::SUCCESS, ck,
										  u.xwords.sort.join(' ')]
									end
								}
							}
							swrite(*msg)
						end
					when /^points(\s(.+)|$)/
						@game.crossout
						if $1.empty? || $2 == @name
							swrite(Message::SUCCESS, ck, points.to_s)
						else
							msg = @game.crossout {|done|
								@game.user_do($2) {|u|
									if !done
										[Message::FAILURE, ck,
										  'game not finished']
									elsif !u
										[Message::FAILURE, ck, 'invalid user']
									else
										[Message::SUCCESS, ck, u.points.to_s]
									end
								}
							}
							swrite(*msg)
						end
					when /^word\s(.+)$/
						w = $1.chomp.downcase
						msg = @game.locked {
							if !@game.board.has_word?(w)
								[Message::FAILURE, ck,
								  'word not found in board']
							elsif @words.include?(w)
								[Message::FAILURE, ck, 'word already recorded']
							elsif !@game.dict.has_word?(w)
								[Message::FAILURE, ck, 'word not recognized']
							else
								if @game.finished?
									[Message::FAILURE, ck, 'game already over']
								else
									@words.push(w)
									[Message::SUCCESS, ck, 'word recorded']
								end
							end
						}
						swrite(*msg)
					when /^say\s([^\s]+)\s(.+)$/
						msg = @game.user_do($1) {|u|
							if !u
								[Message::FAILURE, ck, 'invalid user']
							else
								u.say(@name, $2)
								[Message::SUCCESS, ck, 'message sent']
							end
						}
						swrite(*msg)
					when /^shout\s(.+)$/
						@game.each_user {|u|
							u.shout(@name, $1)
						}
						swrite(Message::SUCCESS, ck, 'message shouted')
					when /^quit(\s(.+)|$)$/
						@game.each_user {|u|
							u.quit(@name, $2 || '')
						}
						swrite(Message::SUCCESS, ck, 'quit sent')
						goodbye('requested by client')
					else
						swrite(Message::FAILURE, ck, 'invalid command')
					end
				end
			end
			def queue_event(event)
				@commandq.push(event)
				@mutex.synchronize {
					@mustrun += 1
					@cvar.signal
				}
			end
			public		# external interfaces
			def wait
				@thread.join
				@selthread.join
			end
			def goodbye(message)
				queue_event([:goodbye, message])
			end
			def quit(from, message)
				queue_event([:quit, from, message])
			end
			def say(from, message)
				queue_event([:say, from, message])
			end
			def shout(from, message)
				queue_event([:shout, from, message])
			end
			def newgame(message)
				queue_event([:newgame, message])
			end
			def timeover(message)
				queue_event([:timeover, message])
			end
		end
		attr_reader :game
		# uses a Hash for options
		def initialize(opts = {})
			@game = Game.new(opts)
			@stop = false
			opts[:host] ||= '0'
			opts[:port] ||= 31782
			@opts = opts
			@socket = TCPServer.new(opts[:host], opts[:port])
			@thread = Thread.new {
				begin
					until @stop
						process_events
					end
					@game.each_user {|user|
						user.goodbye('BonkleServer shut down')
					}
					users = []
					@game.each_user {|user| users.push(user)}
					users.each {|user| user.wait}
					@socket.close
				end
			}
		end
		def stop
			@stop = true
			@thread.join
		end
		def reset
			@game.reset
		end
		def process_events
			if select([@socket], [], [], 0.1)
				begin
					s = @socket.accept
				rescue
					return
				end
				User.new(s, @game)
			end
		end
		private :process_events
	end
	class Game
		attr_reader :dict
		attr_reader :endtime
		attr_reader :mutex
		def initialize(opts = {})
			@board = Bonkle::Board.new
			@dict = Bonkle::Dict.new
			@users = {}
			@mutex = Mutex.new
			opts[:gamelength] ||= 300
			opts[:maxconn] ||= 5
			if !opts[:points]
				pn = Hash.new(11)
				pn[3] = 1
				pn[4] = 1
				pn[5] = 2
				pn[6] = 3
				pn[7] = 5
				pn[8] = 7
				opts[:points] = lambda {|word| pn[word.size]}
			end
			@opts = opts
			@starttime = Time.now
			@endtime = @starttime + opts[:gamelength]
			new_timeover
		end
		def new_timeover
			locked {
				unless @timeover.nil?
					@timeover.kill
				end
			}
			@timeover = Thread.new {
				sleep(@endtime - Time.now)
				locked {
					@users.each {|name, user|
						user.timeover('Round time is up.')
					}
				}
			}
		end
		private :new_timeover
		def locked(&block)
			@mutex.synchronize(&block)
		end
		def reset
			locked {
				@board.reset
				@starttime = Time.now
				@endtime = @starttime + @opts[:gamelength]
				@crossedout = false
				@users.each {|name, user|
					user.words.clear
					user.xwords.clear
					user.newgame('Game board reset.')
				}
			}
			new_timeover
		end
		def crossout
			locked {
				f = finished?
				if f && !@crossedout
					@crossedout = true
					users = @users.collect {|name, user| [user, user.words.dup]}
					users.each {|user|
						users.each {|user2|
							next if user[0].eql?(user2[0])
							user2[1].each {|word2|
								if user[0].words.delete(word2)
									user[0].xwords.push(word2)
								end
							}
						}
					}
				end
				yield(f) if block_given?
			}
		end
		def points(word)
			@opts[:points].call(word)
		end
		def user_do(name)
			yield(locked {@users[name]})
		end
		def add_user(user)
			locked {
				if @users.size == @opts[:maxconn]
					raise(user.type::MaxConn,
					  "maximum number of connections exceeded")
				elsif @users.has_key?(user.name)
					raise(user.type::NameUsed, "user name in use")
				else
					@users[user.name] = user
				end
			}
		end
		def remove_user(user)
			locked {
				@users.delete(user.name)
			}
		end
		def each_user
			locked {@users.each {|name, user| yield user}}
		end
		def usernames
			locked {@users.collect {|name, user| name}}
		end
		def has_word?(word)
			@board.has_word?(word)
		end
		def finished?
			Time.now >= @endtime
		end
		def board
			@board
		end
	end
	class Client
		def initialize(opts = {})
			opts[:host] ||= 'localhost'
			opts[:port] ||= 31782
			@socket = TCPSocket.new(opts[:host], opts[:port])
		end
		def write(msg)
			Message::write(@socket, '0 ' + msg)
		end
		def read
			Message::read(@socket)
		end
		def tryread
			Message::read(@socket) if select([@socket], nil, nil, 0)
		end
		def say(to, msg)
			write("say #{to} #{msg}")
		end
		def shout(msg)
			write("shout #{msg}")
		end
		def name(n)
			write("name #{n}")
		end
		def board
			write("board")
		end
		def boardsize
			write("boardsize")
		end
		def fboard
			write("fboard")
		end
		def timeleft
			write("timeleft")
		end
		def word(w)
			write("word #{w}")
		end
		def lusers
			write("lusers")
		end
		def points(who = nil)
			write("points" + (who ? " " + who : ""))
		end
		def lwords(who = nil)
			write("lwords" + (who ? " " + who : ""))
		end
		def lxwords(who = nil)
			write("lxwords" + (who ? " " + who : ""))
		end
		def quit(message)
			write("quit")
		end
	end
end

==> bonkle-0.10.tar.gz: bonkle/source/bonklenet.rb <==
#!/usr/local/bin/ruby
# Copyright (c) 2001 Brian Fundakowski Feldman
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: bonklenet.rb,v 1.13 2001/03/08 15:46:30 green Exp $
require 'bonkle'
require 'nbio'
require 'socket'
require 'thread'

module Bonkle
	module Message
		def Message.read(socket)
			m = socket.sysread(2)
			if m.size != 2
				return nil
			end
			socket.sysread(m.unpack('n')[0])
		end
		def Message.read_with_left(socket)
			m = socket.sysread(2)
			if m.size != 2
				return [m, nil]
			end
			len = m.unpack('n')[0]
			m = socket.sysread(len)
			[m, len - m.size]
		end
		def Message.complete_read(socket, res)
			if res[1] == nil
				res[0].concat(socket.sysread(2 - res[0].size))
				res[1] = res[0].unpack('n')[0]
				res[0] = ''
			end
			m = socket.sysread(res[1])
			res[1] -= m.size
			res[0].concat(m)
			res[1] == 0
		end
		def Message.write(socket, message)
			socket.syswrite([message.size].pack('n').concat(message))
		end
		SUCCESS = 0
		FAILURE = 1
		FATAL = 2
		@@ecodes = {
			SUCCESS	=> '',			# success
			FAILURE	=> 'error: ',	# failure
			FATAL	=> 'fatal: '	# fatal, close the connection
		}
		def Message.swrite(socket, ecode, cookie, message)
			msg = [cookie, ecode, @@ecodes[ecode]].join(' ').concat(message)
			socket.syswrite([msg.size].pack('n').concat(msg))
		end
	end
	class Server
		class User
			[:name, :words, :xwords, :status, :commandq].each {|att|
			  attr_reader att
			}
			class MaxConn < Exception; end
			class NameUsed < Exception; end
			def initialize(sock, game)
				sock.nbio = true
				@socket = sock
				@game = game
				@name = ''
				@words = []
				@xwords = []
				@rmessages = []
				@status = :init
				@starttime = Time.now
				@commandq = Queue.new
				@stop = false
				@mutex = Mutex.new
				@cvar = ConditionVariable.new
				@mustrun = 0
				@selthread = Thread.new {
					begin
						@rdincomp = false
						until @status == :dead
							sel = select([@socket], [], [], 1)
							if sel
								msg = real_read
								if msg
									@mutex.synchronize {
										@rmessages.push(msg)
										@mustrun += 1
										@cvar.signal
									}
								end
							end
						end
					rescue EOFError => e
						# Will occur if remote disconnects
						goodbye('remote disconnected')
					end
				}
				@thread = Thread.new {
					begin
						write("info Hello from Bonkle #{Bonkle::VERSION}; " + 
						  "please introduce yourself.")
						until @status == :dead
							process_event
						end
					rescue Errno => e
						# This can occur if the remote end isn't reading
						# enough.  I don't have a good solution at the
						# moment.   I'd like to query how much send
						# buffer space is used at the moment...
					ensure
						if @status == :run
							@game.remove_user(self)
						end
						@status = :dead
					end
				}
			end
			def points
				pn = 0
				@game.locked {
					@words.each {|word| pn += @game.points(word)}
				}
				pn
			end
			private
			def real_read
				if @rdincomp
					if Message.complete_read(@socket, @rdincomp)
						ret = @rdincomp[0]
						@rdincomp = nil
						ret
					else
						nil
					end
				else
					res = Message.read_with_left(@socket)
					if res[1] == 0
						res[0]
					else
						@rdincomp = res
						nil
					end
				end
			end
			def read
				@mutex.synchronize {
					msg = @rmessages.shift
					@mustrun -= 1
					msg
				}
			end
			def readable
				@mutex.synchronize {
					!@rmessages.empty?
				}
			end
			def write(*args)
				Message.write(@socket, *args)
			end
			def swrite(*args)
				Message.swrite(@socket, *args)
			end
			def process_event
				@mutex.synchronize {
					if @mustrun == 0
						@cvar.wait(@mutex)
					end
				}

				until @commandq.empty?
					command = @commandq.shift
					case command[0]
					when :info
						write("info #{command[1]}")
					when :say
						write("say #{command[1]} #{command[2]}")
					when :shout
						write("shout #{command[1]} #{command[2]}")
					when :quit
						write("quit #{command[1]} #{command[2]}")
					when :timeover
						write("timeover #{command[1]}")
					when :newgame
						write("newgame #{command[1]}")
					when :goodbye
						begin
							write("goodbye " + command[1])
						ensure
							@socket.close
							if @status != :init
								@game.remove_user(self)
							end
							@status = :dead
							return
						end
					end
					@mutex.synchronize {@mustrun -= 1}
				end

				if !readable
					return
				end

				case @status
				when :init
					message = read
					if message =~ /^(\d+)\sname\s(.+)$/
					    ck = $1
						@name = $2
						if @name !~ /^[A-Za-z0-9_-]*$/ || @name.length > 20
							@name = ''
						end
					end
					if @name.empty?
						swrite(Message::FAILURE, ck, "need valid name")
						return
					else
						begin
							@game.add_user(self)
						rescue MaxConn => e
							swrite(Message::FAILURE, ck, e.to_s)
							@stop = true
							return
						rescue NameUsed => e
							swrite(Message::FAILURE, ck, e.to_s)
							return
						end
					end
					swrite(Message::SUCCESS, ck, "name set")
					@status = :run
				when :run
					message = read
					if message !~ /^(\d+)\s/
						swrite(Message::FATAL, 0, 'invalid command')
						goodbye('fatal protocol error encountered')
						return
					end
					ck = $1		# cookie will be used
					case $'
					when /^timeleft(\s|$)/
						swrite(Message::SUCCESS, ck,
						  format("%.4f", @game.endtime - Time.now))
					when /^fboard(\s|$)/
						swrite(Message::SUCCESS, ck,
						  "pre-formatted Bonkle board:\n" +
						  @game.locked {@game.board.to_s})
					when /^boardsize(\s|$)/
						swrite(Message::SUCCESS, ck, @game.board.width.to_s)
					when /^board(\s|$)/
						swrite(Message::SUCCESS, ck,
						  @game.locked {@game.board.dice_array.join(' ')})
					when /^lusers(\s|$)/
						swrite(Message::SUCCESS, ck, @game.usernames.join(' '))
					when /^lwords(\s(.+)|$)/
						@game.crossout
						if $1.empty? || $2 == @name
							swrite(Message::SUCCESS, ck,
							  @game.locked {@words.sort.join(' ')})
						else
							msg = @game.crossout {|done|
								@game.user_do($2) {|u|
									if !done
										[Message::FAILURE, ck,
										  'game not finished']
									elsif !u
										[Message::FAILURE, ck, 'invalid user']
									else
										[Message::SUCCESS, ck,
										  u.words.sort.join(' ')]
									end
								}
							}
							swrite(*msg)
						end
					when /^lxwords(\s(.+)|$)/
						@game.crossout
						if $1.empty? || $2 == @name
							swrite(Message::SUCCESS, ck,
							  @game.locked {@xwords.sort.join(' ')})
						else
							msg = @game.crossout {|done|
								@game.user_do($2) {|u|
									if !done
										[Message::FAILURE, ck,
										  'game not finished']
									elsif !u
										[Message::FAILURE, ck, 'invalid user']
									else
										[Message::SUCCESS, ck,
										  u.xwords.sort.join(' ')]
									end
								}
							}
							swrite(*msg)
						end
					when /^points(\s(.+)|$)/
						@game.crossout
						if $1.empty? || $2 == @name
							swrite(Message::SUCCESS, ck, points.to_s)
						else
							msg = @game.crossout {|done|
								@game.user_do($2) {|u|
									if !done
										[Message::FAILURE, ck,
										  'game not finished']
									elsif !u
										[Message::FAILURE, ck, 'invalid user']
									else
										[Message::SUCCESS, ck, u.points.to_s]
									end
								}
							}
							swrite(*msg)
						end
					when /^word\s(.+)$/
						w = $1.chomp.downcase
						msg = @game.locked {
							if !@game.board.has_word?(w)
								[Message::FAILURE, ck,
								  'word not found in board']
							elsif @words.include?(w)
								[Message::FAILURE, ck, 'word already recorded']
							elsif !@game.dict.has_word?(w)
								[Message::FAILURE, ck, 'word not recognized']
							else
								if @game.finished?
									[Message::FAILURE, ck, 'game already over']
								else
									@words.push(w)
									[Message::SUCCESS, ck, 'word recorded']
								end
							end
						}
						swrite(*msg)
					when /^say\s([^\s]+)\s(.+)$/
						msg = @game.user_do($1) {|u|
							if !u
								[Message::FAILURE, ck, 'invalid user']
							else
								u.say(@name, $2)
								[Message::SUCCESS, ck, 'message sent']
							end
						}
						swrite(*msg)
					when /^shout\s(.+)$/
						@game.each_user {|u|
							u.shout(@name, $1)
						}
						swrite(Message::SUCCESS, ck, 'message shouted')
					when /^quit(\s(.+)|$)$/
						@game.each_user {|u|
							u.quit(@name, $2 || '')
						}
						swrite(Message::SUCCESS, ck, 'quit sent')
						goodbye('requested by client')
					else
						swrite(Message::FAILURE, ck, 'invalid command')
					end
				end
			end
			def queue_event(event)
				@commandq.push(event)
				@mutex.synchronize {
					@mustrun += 1
					@cvar.signal
				}
			end
			public		# external interfaces
			def wait
				@thread.join
				@selthread.join
			end
			def goodbye(message)
				queue_event([:goodbye, message])
			end
			def quit(from, message)
				queue_event([:quit, from, message])
			end
			def say(from, message)
				queue_event([:say, from, message])
			end
			def shout(from, message)
				queue_event([:shout, from, message])
			end
			def newgame(message)
				queue_event([:newgame, message])
			end
			def timeover(message)
				queue_event([:timeover, message])
			end
		end
		attr_reader :game
		# uses a Hash for options
		def initialize(opts = {})
			@game = Game.new(opts)
			@stop = false
			opts[:host] ||= '0'
			opts[:port] ||= 31782
			@opts = opts
			@socket = TCPServer.new(opts[:host], opts[:port])
			@thread = Thread.new {
				begin
					until @stop
						process_events
					end
					@game.each_user {|user|
						user.goodbye('BonkleServer shut down')
					}
					users = []
					@game.each_user {|user| users.push(user)}
					users.each {|user| user.wait}
					@socket.close
				end
			}
		end
		def stop
			@stop = true
			@thread.join
		end
		def reset
			@game.reset
		end
		def process_events
			if select([@socket], [], [], 1)
				begin
					s = @socket.accept
				rescue
					return
				end
				User.new(s, @game)
			end
		end
		private :process_events
	end
	class Game
		attr_reader :dict
		attr_reader :endtime
		attr_reader :mutex
		def initialize(opts = {})
			@board = Bonkle::Board.new
			@dict = Bonkle::Dict.new
			@users = {}
			@mutex = Mutex.new
			opts[:gamelength] ||= 300
			opts[:maxconn] ||= 5
			if !opts[:points]
				pn = Hash.new(11)
				pn[3] = 1
				pn[4] = 1
				pn[5] = 2
				pn[6] = 3
				pn[7] = 5
				pn[8] = 7
				opts[:points] = lambda {|word| pn[word.size]}
			end
			@opts = opts
			@starttime = Time.now
			@endtime = @starttime + opts[:gamelength]
			new_timeover
		end
		def new_timeover
			locked {
				unless @timeover.nil?
					@timeover.kill
				end
			}
			@timeover = Thread.new {
				sleep(@endtime - Time.now)
				locked {
					@users.each {|name, user|
						user.timeover('Round time is up.')
					}
				}
			}
		end
		private :new_timeover
		def locked(&block)
			@mutex.synchronize(&block)
		end
		def reset
			locked {
				@board.reset
				@starttime = Time.now
				@endtime = @starttime + @opts[:gamelength]
				@crossedout = false
				@users.each {|name, user|
					user.words.clear
					user.xwords.clear
					user.newgame('Game board reset.')
				}
			}
			new_timeover
		end
		def crossout
			locked {
				f = finished?
				if f && !@crossedout
					@crossedout = true
					users = @users.collect {|name, user| [user, user.words.dup]}
					users.each {|user|
						users.each {|user2|
							next if user[0].eql?(user2[0])
							user2[1].each {|word2|
								if user[0].words.delete(word2)
									user[0].xwords.push(word2)
								end
							}
						}
					}
				end
				yield(f) if block_given?
			}
		end
		def points(word)
			@opts[:points].call(word)
		end
		def user_do(name)
			yield(locked {@users[name]})
		end
		def add_user(user)
			locked {
				if @users.size == @opts[:maxconn]
					raise(user.type::MaxConn,
					  "maximum number of connections exceeded")
				elsif @users.has_key?(user.name)
					raise(user.type::NameUsed, "user name in use")
				else
					@users[user.name] = user
				end
			}
		end
		def remove_user(user)
			locked {
				@users.delete(user.name)
			}
		end
		def each_user
			locked {@users.each {|name, user| yield user}}
		end
		def usernames
			locked {@users.collect {|name, user| name}}
		end
		def has_word?(word)
			@board.has_word?(word)
		end
		def finished?
			Time.now >= @endtime
		end
		def board
			@board
		end
	end
	class Client
		def initialize(opts = {})
			opts[:host] ||= 'localhost'
			opts[:port] ||= 31782
			@socket = TCPSocket.new(opts[:host], opts[:port])
		end
		def write(msg)
			Message::write(@socket, '0 ' + msg)
		end
		def read
			Message::read(@socket)
		end
		def tryread
			Message::read(@socket) if select([@socket], nil, nil, 0)
		end
		def say(to, msg)
			write("say #{to} #{msg}")
		end
		def shout(msg)
			write("shout #{msg}")
		end
		def name(n)
			write("name #{n}")
		end
		def board
			write("board")
		end
		def boardsize
			write("boardsize")
		end
		def fboard
			write("fboard")
		end
		def timeleft
			write("timeleft")
		end
		def word(w)
			write("word #{w}")
		end
		def lusers
			write("lusers")
		end
		def points(who = nil)
			write("points" + (who ? " " + who : ""))
		end
		def lwords(who = nil)
			write("lwords" + (who ? " " + who : ""))
		end
		def lxwords(who = nil)
			write("lxwords" + (who ? " " + who : ""))
		end
		def quit(message)
			write("quit")
		end
	end
end
